@@ -2,15 +2,16 @@ package kubeletconfig
 
 import (
 	"bytes"
+	"context"
 	"encoding/json"
 	"fmt"
+	"strconv"
 
-	ign3types "github.com/coreos/ignition/v2/config/v3_1/types"
+	ign3types "github.com/coreos/ignition/v2/config/v3_2/types"
 	osev1 "github.com/openshift/api/config/v1"
 	"github.com/vincent-petithory/dataurl"
 	corev1 "k8s.io/api/core/v1"
-	v1 "k8s.io/api/core/v1"
-	"k8s.io/apimachinery/pkg/api/resource"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/runtime/serializer"
@@ -22,7 +23,30 @@ import (
 	mcfgclientset "github.com/openshift/machine-config-operator/pkg/generated/clientset/versioned"
 )
 
-func createNewKubeletIgnition(jsonConfig []byte) ign3types.Config {
+func createNewKubeletLogLevelIgnition(level int32) *ign3types.File {
+	config := fmt.Sprintf("[Service]\nEnvironment=\"KUBELET_LOG_LEVEL=%d\"\n", level)
+
+	mode := 0644
+	overwrite := true
+	du := dataurl.New([]byte(config), "text/plain")
+	du.Encoding = dataurl.EncodingASCII
+	duStr := du.String()
+
+	return &ign3types.File{
+		Node: ign3types.Node{
+			Path:      "/etc/systemd/system/kubelet.service.d/20-logging.conf",
+			Overwrite: &overwrite,
+		},
+		FileEmbedded1: ign3types.FileEmbedded1{
+			Mode: &mode,
+			Contents: ign3types.Resource{
+				Source: &(duStr),
+			},
+		},
+	}
+}
+
+func createNewKubeletIgnition(jsonConfig []byte) *ign3types.File {
 	// Want the kubelet.conf file to have the pretty JSON formatting
 	buf := new(bytes.Buffer)
 	json.Indent(buf, jsonConfig, "", "  ")
@@ -32,7 +56,8 @@ func createNewKubeletIgnition(jsonConfig []byte) ign3types.Config {
 	du := dataurl.New(buf.Bytes(), "text/plain")
 	du.Encoding = dataurl.EncodingASCII
 	duStr := du.String()
-	tempFile := ign3types.File{
+
+	return &ign3types.File{
 		Node: ign3types.Node{
 			Path:      "/etc/kubernetes/kubelet.conf",
 			Overwrite: &overwrite,
@@ -44,13 +69,13 @@ func createNewKubeletIgnition(jsonConfig []byte) ign3types.Config {
 			},
 		},
 	}
-	tempIgnConfig := ctrlcommon.NewIgnConfig()
-	tempIgnConfig.Storage.Files = append(tempIgnConfig.Storage.Files, tempFile)
-	return tempIgnConfig
 }
 
 func createNewDefaultFeatureGate() *osev1.FeatureGate {
 	return &osev1.FeatureGate{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: clusterFeatureInstanceName,
+		},
 		Spec: osev1.FeatureGateSpec{
 			FeatureGateSelection: osev1.FeatureGateSelection{
 				FeatureSet: osev1.Default,
@@ -72,8 +97,56 @@ func findKubeletConfig(mc *mcfgv1.MachineConfig) (*ign3types.File, error) {
 	return nil, fmt.Errorf("Could not find Kubelet Config")
 }
 
-func getManagedKubeletConfigKey(pool *mcfgv1.MachineConfigPool, client mcfgclientset.Interface) (string, error) {
-	return ctrlcommon.GetManagedKey(pool, client, "99", "kubelet", getManagedKubeletConfigKeyDeprecated(pool))
+// nolint: dupl
+func getManagedKubeletConfigKey(pool *mcfgv1.MachineConfigPool, client mcfgclientset.Interface, cfg *mcfgv1.KubeletConfig) (string, error) {
+	// Get all the kubelet config CRs
+	kcList, err := client.MachineconfigurationV1().KubeletConfigs().List(context.TODO(), metav1.ListOptions{})
+	if err != nil {
+		return "", fmt.Errorf("error listing kubelet configs: %v", err)
+	}
+	// If there is no kubelet config in the list, return the default MC name with no suffix
+	if kcList == nil || len(kcList.Items) == 0 {
+		return ctrlcommon.GetManagedKey(pool, client, "99", "kubelet", getManagedKubeletConfigKeyDeprecated(pool))
+	}
+	for _, kc := range kcList.Items {
+		if kc.Name != cfg.Name {
+			continue
+		}
+		val, ok := kc.GetAnnotations()[ctrlcommon.MCNameSuffixAnnotationKey]
+		// If we find a matching kubelet config and it is the only one in the list, then return the default MC name with no suffix
+		if !ok && len(kcList.Items) < 9 {
+			return ctrlcommon.GetManagedKey(pool, client, "99", "kubelet", getManagedKubeletConfigKeyDeprecated(pool))
+		}
+		// Otherwise if an MC name suffix exists, append it to the default MC name and return that as this kubelet config exists and
+		// we are probably doing an update action on it
+		if val != "" {
+			return fmt.Sprintf("99-%s-generated-kubelet-%s", pool.Name, val), nil
+		}
+	}
+
+	// If we are here, this means that a new kubelet config was created, so we have to calculate the suffix value for its MC name
+	suffixNum := 0
+	// Get the suffix value of the second to last item in the kubelet config list. We use the second to last item because
+	// the most recent kubelet config being created has already been added to the list and is the kubelet config object that we are
+	// trying to figure out the suffix value for
+	val, ok := kcList.Items[len(kcList.Items)-2].GetAnnotations()[ctrlcommon.MCNameSuffixAnnotationKey]
+	if ok {
+		// Convert the suffix value to int so we can add 1 to it for the MC name for the latest kubelet config object
+		suffixNum, err = strconv.Atoi(val)
+		if err != nil {
+			return "", fmt.Errorf("error converting %s to int: %v", val, err)
+		}
+	}
+	// The max suffix value that we can go till with this logic is 9 - this means that a user can create up to 10 different kubelet config CRs.
+	// However, if there is a kc-1 mapping to mc-1 and kc-2 mapping to mc-2 and the user deletes kc-1, it will delete mc-1 but
+	// then if the user creates a kc-new it will map to mc-3. This is what we want as the latest kubelet config created should be higher in priority
+	// so that those changes can be rolled out to the nodes. But users will have to be mindful of how many kubelet config CRs they create. Don't think
+	// anyone should ever have the need to create 10 when they can simply update an existing kubelet config unless it is to apply to another pool.
+	if suffixNum+1 > 2 {
+		return "", fmt.Errorf("max number of supported kubelet config (10) has been reached. Please delete old kubelet configs before retrying")
+	}
+	// Return the default MC name with the suffixNum+1 value appended to it
+	return fmt.Sprintf("99-%s-generated-kubelet-%s", pool.Name, strconv.Itoa(suffixNum+1)), nil
 }
 
 func getManagedFeaturesKey(pool *mcfgv1.MachineConfigPool, client mcfgclientset.Interface) (string, error) {
@@ -91,8 +164,12 @@ func getManagedKubeletConfigKeyDeprecated(pool *mcfgv1.MachineConfigPool) string
 }
 
 // validates a KubeletConfig and returns an error if invalid
+// nolint:gocyclo
 func validateUserKubeletConfig(cfg *mcfgv1.KubeletConfig) error {
-	if cfg.Spec.KubeletConfig.Raw == nil {
+	if cfg.Spec.LogLevel != nil && (*cfg.Spec.LogLevel < 1 || *cfg.Spec.LogLevel > 10) {
+		return fmt.Errorf("KubeletConfig's LogLevel is not valid [1,10]: %v", cfg.Spec.LogLevel)
+	}
+	if cfg.Spec.KubeletConfig == nil || cfg.Spec.KubeletConfig.Raw == nil {
 		return nil
 	}
 	kcDecoded, err := decodeKubeletConfig(cfg.Spec.KubeletConfig.Raw)
@@ -124,48 +201,6 @@ func validateUserKubeletConfig(cfg *mcfgv1.KubeletConfig) error {
 		return fmt.Errorf("KubeletConfiguration: staticPodPath is not allowed to be set, but contains: %s", kcDecoded.StaticPodPath)
 	}
 
-	if kcDecoded.EvictionSoft != nil && len(kcDecoded.EvictionSoft) > 0 {
-		if kcDecoded.EvictionSoftGracePeriod == nil || len(kcDecoded.EvictionSoftGracePeriod) == 0 {
-			return fmt.Errorf("KubeletConfiguration: EvictionSoftGracePeriod must be set when evictionSoft is defined, evictionSoft: %v", kcDecoded.EvictionSoft)
-		}
-
-		for k := range kcDecoded.EvictionSoft {
-			if _, ok := kcDecoded.EvictionSoftGracePeriod[k]; !ok {
-				return fmt.Errorf("KubeletConfiguration: evictionSoft[%s] is defined but EvictionSoftGracePeriod[%s] is not set", k, k)
-			}
-		}
-	}
-
-	reservedResources := []v1.ResourceName{v1.ResourceCPU, v1.ResourceMemory, v1.ResourceEphemeralStorage}
-
-	if kcDecoded.KubeReserved != nil && len(kcDecoded.KubeReserved) > 0 {
-		for _, rr := range reservedResources {
-			if val, ok := kcDecoded.KubeReserved[rr.String()]; ok {
-				q, err := resource.ParseQuantity(val)
-				if err != nil {
-					return fmt.Errorf("KubeletConfiguration: invalid value specified for %s reservation in kubeReserved, %s", rr.String(), val)
-				}
-				if q.Sign() == -1 {
-					return fmt.Errorf("KubeletConfiguration: %s reservation value cannot be negative in kubeReserved", rr.String())
-				}
-			}
-		}
-	}
-
-	if kcDecoded.SystemReserved != nil && len(kcDecoded.SystemReserved) > 0 {
-		for _, rr := range reservedResources {
-			if val, ok := kcDecoded.SystemReserved[rr.String()]; ok {
-				q, err := resource.ParseQuantity(val)
-				if err != nil {
-					return fmt.Errorf("KubeletConfiguration: invalid value specified for %s reservation in systemReserved, %s", rr.String(), val)
-				}
-				if q.Sign() == -1 {
-					return fmt.Errorf("KubeletConfiguration: %s reservation value cannot be negative in systemReserved", rr.String())
-				}
-			}
-		}
-	}
-
 	return nil
 }
 
@@ -187,7 +222,7 @@ func wrapErrorWithCondition(err error, args ...interface{}) mcfgv1.KubeletConfig
 	if len(args) > 0 {
 		format, ok := args[0].(string)
 		if ok {
-			condition.Message = fmt.Sprintf(format, args[:1]...)
+			condition.Message = fmt.Sprintf(format, args[1:]...)
 		}
 	}
 	return *condition
@@ -202,7 +237,7 @@ func decodeKubeletConfig(data []byte) (*kubeletconfigv1beta1.KubeletConfiguratio
 	return config, nil
 }
 
-func encodeKubeletConfig(internal *kubeletconfigv1beta1.KubeletConfiguration, targetVersion schema.GroupVersion) ([]byte, error) {
+func EncodeKubeletConfig(internal *kubeletconfigv1beta1.KubeletConfiguration, targetVersion schema.GroupVersion) ([]byte, error) {
 	encoder, err := newKubeletconfigJSONEncoder(targetVersion)
 	if err != nil {
 		return nil, err
