@@ -19,15 +19,17 @@ import (
 	apiextv1beta1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/labels"
+	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/apimachinery/pkg/util/wait"
 	"k8s.io/client-go/tools/cache"
 
 	configv1 "github.com/openshift/api/config/v1"
-	operatorv1 "github.com/openshift/api/operator/v1"
 	"github.com/openshift/machine-config-operator/lib/resourceapply"
 	"github.com/openshift/machine-config-operator/lib/resourceread"
 	mcfgv1 "github.com/openshift/machine-config-operator/pkg/apis/machineconfiguration.openshift.io/v1"
 	templatectrl "github.com/openshift/machine-config-operator/pkg/controller/template"
+	daemonconsts "github.com/openshift/machine-config-operator/pkg/daemon/constants"
 	"github.com/openshift/machine-config-operator/pkg/operator/assets"
 	"github.com/openshift/machine-config-operator/pkg/version"
 )
@@ -36,6 +38,15 @@ const (
 	requiredForUpgradeMachineConfigPoolLabelKey = "operator.machineconfiguration.openshift.io/required-for-upgrade"
 )
 
+var (
+	platformsRequiringCloudConf = sets.NewString(
+		string(configv1.AzurePlatformType),
+		string(configv1.GCPPlatformType),
+		string(configv1.OpenStackPlatformType),
+		string(configv1.VSpherePlatformType),
+	)
+)
+
 type syncFunc struct {
 	name string
 	fn   func(config *renderConfig) error
@@ -97,57 +108,53 @@ func (optr *Operator) syncAll(syncFuncs []syncFunc) error {
 	return syncErr.err
 }
 
-// Return true if cloud config is required on a platform.
-func isCloudConfigRequired(infra *configv1.Infrastructure) bool {
-	if infra.Spec.CloudConfig.Name != "" {
-		return true
-	}
+// Return true if kube-cloud-config ConfigMap is required.
+func isKubeCloudConfigCMRequired(infra *configv1.Infrastructure) bool {
+	return infra.Spec.CloudConfig.Name != "" || isCloudConfRequired(infra)
+}
 
-	if infra.Status.PlatformStatus != nil {
-		switch infra.Status.PlatformStatus.Type {
-		case configv1.AzurePlatformType,
-			configv1.GCPPlatformType,
-			configv1.OpenStackPlatformType,
-			configv1.VSpherePlatformType:
-			return true
-		default:
-			return false
-		}
-	} else {
+// Return true if the platform requires a cloud.conf.
+func isCloudConfRequired(infra *configv1.Infrastructure) bool {
+	if infra.Status.PlatformStatus == nil {
 		return false
 	}
+	return platformsRequiringCloudConf.Has(string(infra.Status.PlatformStatus.Type))
 }
 
 // Sync cloud config on supported platform from cloud.conf available in openshift-config-managed/kube-cloud-config ConfigMap.
 func (optr *Operator) syncCloudConfig(spec *mcfgv1.ControllerConfigSpec, infra *configv1.Infrastructure) error {
-	if _, err := optr.clusterCmLister.ConfigMaps("openshift-config-managed").Get("kube-cloud-config"); err != nil {
+	cm, err := optr.clusterCmLister.ConfigMaps("openshift-config-managed").Get("kube-cloud-config")
+	if err != nil {
 		if apierrors.IsNotFound(err) {
-			if isCloudConfigRequired(infra) {
-				// Return error only if cloud config is required, otherwise prceeds further.
+			if isKubeCloudConfigCMRequired(infra) {
+				// Return error only if the kube-cloud-config ConfigMap is required, otherwise proceeds further.
 				return fmt.Errorf("%s/%s configmap is required on platform %s but not found: %v",
 					"openshift-config-managed", "kube-cloud-config", infra.Status.PlatformStatus.Type, err)
 			}
-		} else {
-			return err
+			return nil
+		}
+		return err
+	}
+	// Read cloud.conf from openshift-config-managed/kube-cloud-config ConfigMap.
+	cc, err := getCloudConfigFromConfigMap(cm, "cloud.conf")
+	if err != nil {
+		if isCloudConfRequired(infra) {
+			// Return error only if cloud.conf is required, otherwise proceeds further.
+			return fmt.Errorf("%s/%s configmap must have the %s key on platform %s but not found",
+				"openshift-config-managed", "kube-cloud-config", "cloud.conf", infra.Status.PlatformStatus.Type)
 		}
-
 	} else {
-		// Read cloud.conf from openshift-config-managed/kube-cloud-config ConfigMap.
-		cc, err := optr.getCloudConfigFromConfigMap("openshift-config-managed", "kube-cloud-config", "cloud.conf")
-		if err != nil {
-			return err
-		}
-
 		spec.CloudProviderConfig = cc
+	}
 
-		caCert, err := optr.getCAsFromConfigMap("openshift-config-managed", "kube-cloud-config", "ca-bundle.pem")
-		if err == nil {
-			spec.CloudProviderCAData = caCert
-		}
+	caCert, err := getCAsFromConfigMap(cm, "ca-bundle.pem")
+	if err == nil {
+		spec.CloudProviderCAData = caCert
 	}
 	return nil
 }
 
+//nolint:gocyclo
 func (optr *Operator) syncRenderConfig(_ *renderConfig) error {
 	if err := optr.syncCustomResourceDefinitions(); err != nil {
 		return err
@@ -178,14 +185,6 @@ func (optr *Operator) syncRenderConfig(_ *renderConfig) error {
 	}
 
 	// sync up CAs
-	etcdCA, err := optr.getCAsFromConfigMap("openshift-config", "etcd-serving-ca", "ca-bundle.crt")
-	if err != nil {
-		return err
-	}
-	etcdMetricCA, err := optr.getCAsFromConfigMap("openshift-config", "etcd-metric-serving-ca", "ca-bundle.crt")
-	if err != nil {
-		return err
-	}
 	rootCA, err := optr.getCAsFromConfigMap("kube-system", "root-ca", "ca.crt")
 	if err != nil {
 		return err
@@ -219,11 +218,11 @@ func (optr *Operator) syncRenderConfig(_ *renderConfig) error {
 	imgs.MachineOSContent = osimageurl
 
 	// sync up the ControllerConfigSpec
-	infra, network, proxy, err := optr.getGlobalConfig()
+	infra, network, proxy, dns, err := optr.getGlobalConfig()
 	if err != nil {
 		return err
 	}
-	spec, err := createDiscoveredControllerConfigSpec(infra, network, proxy)
+	spec, err := createDiscoveredControllerConfigSpec(infra, network, proxy, dns)
 	if err != nil {
 		return err
 	}
@@ -261,29 +260,20 @@ func (optr *Operator) syncRenderConfig(_ *renderConfig) error {
 		return err
 	}
 
-	//TODO: alaypatel07 remove after cluster-etcd-operator deployed via CVO as Managed
-	if err = optr.setEtcdOperatorImage(&imgs); err != nil {
-		glog.Errorf("error setting etcd operator images: %#v", err)
-	}
-
 	spec.KubeAPIServerServingCAData = kubeAPIServerServingCABytes
-	spec.EtcdCAData = etcdCA
-	spec.EtcdMetricCAData = etcdMetricCA
 	spec.RootCAData = bundle
 	spec.PullSecret = &corev1.ObjectReference{Namespace: "openshift-config", Name: "pull-secret"}
 	spec.OSImageURL = imgs.MachineOSContent
 	spec.Images = map[string]string{
-		templatectrl.EtcdImageKey:                imgs.Etcd,
-		templatectrl.SetupEtcdEnvKey:             imgs.MachineConfigOperator,
-		templatectrl.GCPRoutesControllerKey:      imgs.MachineConfigOperator,
-		templatectrl.InfraImageKey:               imgs.InfraImage,
-		templatectrl.KubeClientAgentImageKey:     imgs.KubeClientAgent,
-		templatectrl.ClusterEtcdOperatorImageKey: imgs.ClusterEtcdOperator,
-		templatectrl.KeepalivedKey:               imgs.Keepalived,
-		templatectrl.CorednsKey:                  imgs.Coredns,
-		templatectrl.MdnsPublisherKey:            imgs.MdnsPublisher,
-		templatectrl.HaproxyKey:                  imgs.Haproxy,
-		templatectrl.BaremetalRuntimeCfgKey:      imgs.BaremetalRuntimeCfg,
+		templatectrl.MachineConfigOperatorKey: imgs.MachineConfigOperator,
+
+		templatectrl.APIServerWatcherKey:    imgs.MachineConfigOperator,
+		templatectrl.InfraImageKey:          imgs.InfraImage,
+		templatectrl.KeepalivedKey:          imgs.Keepalived,
+		templatectrl.CorednsKey:             imgs.Coredns,
+		templatectrl.MdnsPublisherKey:       imgs.MdnsPublisher,
+		templatectrl.HaproxyKey:             imgs.Haproxy,
+		templatectrl.BaremetalRuntimeCfgKey: imgs.BaremetalRuntimeCfg,
 	}
 
 	// create renderConfig
@@ -338,14 +328,34 @@ func (optr *Operator) syncMachineConfigPools(config *renderConfig) error {
 }
 
 func (optr *Operator) syncMachineConfigController(config *renderConfig) error {
-	crBytes, err := renderAsset(config, "manifests/machineconfigcontroller/clusterrole.yaml")
-	if err != nil {
-		return err
+	for _, path := range []string{
+		"manifests/machineconfigcontroller/clusterrole.yaml",
+		"manifests/machineconfigcontroller/events-clusterrole.yaml",
+	} {
+		crBytes, err := renderAsset(config, path)
+		if err != nil {
+			return err
+		}
+		cr := resourceread.ReadClusterRoleV1OrDie(crBytes)
+		_, _, err = resourceapply.ApplyClusterRole(optr.kubeClient.RbacV1(), cr)
+		if err != nil {
+			return err
+		}
 	}
-	cr := resourceread.ReadClusterRoleV1OrDie(crBytes)
-	_, _, err = resourceapply.ApplyClusterRole(optr.kubeClient.RbacV1(), cr)
-	if err != nil {
-		return err
+
+	for _, path := range []string{
+		"manifests/machineconfigcontroller/events-rolebinding-default.yaml",
+		"manifests/machineconfigcontroller/events-rolebinding-target.yaml",
+	} {
+		crbBytes, err := renderAsset(config, path)
+		if err != nil {
+			return err
+		}
+		crb := resourceread.ReadRoleBindingV1OrDie(crbBytes)
+		_, _, err = resourceapply.ApplyRoleBinding(optr.kubeClient.RbacV1(), crb)
+		if err != nil {
+			return err
+		}
 	}
 
 	crbBytes, err := renderAsset(config, "manifests/machineconfigcontroller/clusterrolebinding.yaml")
@@ -388,6 +398,11 @@ func (optr *Operator) syncMachineConfigController(config *renderConfig) error {
 		return err
 	}
 	cc := resourceread.ReadControllerConfigV1OrDie(ccBytes)
+	// Propagate our binary version into the controller config to help
+	// suppress rendered config generation until a corresponding
+	// new controller can roll out too.
+	// https://bugzilla.redhat.com/show_bug.cgi?id=1879099
+	cc.Annotations[daemonconsts.GeneratedByVersionAnnotationKey] = version.Raw
 	_, _, err = resourceapply.ApplyControllerConfig(optr.client.MachineconfigurationV1(), cc)
 	if err != nil {
 		return err
@@ -552,18 +567,7 @@ func (optr *Operator) syncMachineConfigServer(config *renderConfig) error {
 // syncRequiredMachineConfigPools ensures that all the nodes in machineconfigpools labeled with requiredForUpgradeMachineConfigPoolLabelKey
 // have updated to the latest configuration.
 func (optr *Operator) syncRequiredMachineConfigPools(_ *renderConfig) error {
-	sel, err := metav1.LabelSelectorAsSelector(metav1.AddLabelToSelector(&metav1.LabelSelector{}, requiredForUpgradeMachineConfigPoolLabelKey, ""))
-	if err != nil {
-		return err
-	}
-	isPoolStatusConditionTrue := func(pool *mcfgv1.MachineConfigPool, conditionType mcfgv1.MachineConfigPoolConditionType) bool {
-		for _, condition := range pool.Status.Conditions {
-			if condition.Type == conditionType {
-				return condition.Status == corev1.ConditionTrue
-			}
-		}
-		return false
-	}
+	glog.Infof("syncing Required MachineConfigPools")
 
 	var lastErr error
 	if err := wait.Poll(time.Second, 10*time.Minute, func() (bool, error) {
@@ -584,28 +588,48 @@ func (optr *Operator) syncRequiredMachineConfigPools(_ *renderConfig) error {
 				return false, nil
 			}
 		}
-		pools, err := optr.mcpLister.List(sel)
+
+		pools, err := optr.mcpLister.List(labels.Everything())
 		if err != nil {
 			lastErr = err
 			return false, nil
 		}
 		for _, pool := range pools {
-			if err := isMachineConfigPoolConfigurationValid(pool, version.Hash, optr.mcLister.Get); err != nil {
-				lastErr = fmt.Errorf("pool %s has not progressed to latest configuration: %v, retrying", pool.Name, err)
+			degraded := isPoolStatusConditionTrue(pool, mcfgv1.MachineConfigPoolDegraded)
+			if degraded {
+				lastErr = fmt.Errorf("error pool %s is not ready, retrying. Status: (pool degraded: %v total: %d, ready %d, updated: %d, unavailable: %d)", pool.Name, degraded, pool.Status.MachineCount, pool.Status.ReadyMachineCount, pool.Status.UpdatedMachineCount, pool.Status.UnavailableMachineCount)
+				glog.Errorf("Error syncing Required MachineConfigPools: %q", lastErr)
+				syncerr := optr.syncUpgradeableStatus()
+				if syncerr != nil {
+					glog.Errorf("Error syncingUpgradeableStatus: %q", syncerr)
+				}
 				return false, nil
 			}
-			degraded := isPoolStatusConditionTrue(pool, mcfgv1.MachineConfigPoolDegraded)
-			if pool.Generation <= pool.Status.ObservedGeneration &&
-				isPoolStatusConditionTrue(pool, mcfgv1.MachineConfigPoolUpdated) &&
-				!degraded {
-				continue
+
+			_, hasRequiredPoolLabel := pool.Labels[requiredForUpgradeMachineConfigPoolLabelKey]
+
+			if hasRequiredPoolLabel {
+				if err := isMachineConfigPoolConfigurationValid(pool, version.Hash, optr.mcLister.Get); err != nil {
+					lastErr = fmt.Errorf("pool %s has not progressed to latest configuration: %v, retrying", pool.Name, err)
+					return false, nil
+				}
+
+				if pool.Generation <= pool.Status.ObservedGeneration &&
+					isPoolStatusConditionTrue(pool, mcfgv1.MachineConfigPoolUpdated) {
+					continue
+				}
+				lastErr = fmt.Errorf("error required pool %s is not ready, retrying. Status: (total: %d, ready %d, updated: %d, unavailable: %d, degraded: %d)", pool.Name, pool.Status.MachineCount, pool.Status.ReadyMachineCount, pool.Status.UpdatedMachineCount, pool.Status.UnavailableMachineCount, pool.Status.DegradedMachineCount)
+				return false, nil
 			}
-			lastErr = fmt.Errorf("error pool %s is not ready, retrying. Status: (pool degraded: %v total: %d, ready %d, updated: %d, unavailable: %d)", pool.Name, degraded, pool.Status.MachineCount, pool.Status.ReadyMachineCount, pool.Status.UpdatedMachineCount, pool.Status.UnavailableMachineCount)
-			return false, nil
+		}
+		syncstatuserr := optr.syncUpgradeableStatus()
+		if syncstatuserr != nil {
+			glog.Errorf("Error syncingUpgradeableStatus: %q", syncstatuserr)
 		}
 		return true, nil
 	}); err != nil {
 		if err == wait.ErrWaitTimeout {
+			glog.Errorf("Error syncing Required MachineConfigPools: %q", lastErr)
 			return fmt.Errorf("%v during syncRequiredMachineConfigPools: %v", err, lastErr)
 		}
 		return err
@@ -755,6 +779,10 @@ func (optr *Operator) getCAsFromConfigMap(namespace, name, key string) ([]byte,
 	if err != nil {
 		return nil, err
 	}
+	return getCAsFromConfigMap(cm, key)
+}
+
+func getCAsFromConfigMap(cm *corev1.ConfigMap, key string) ([]byte, error) {
 	if bd, bdok := cm.BinaryData[key]; bdok {
 		return bd, nil
 	} else if d, dok := cm.Data[key]; dok {
@@ -766,7 +794,7 @@ func (optr *Operator) getCAsFromConfigMap(namespace, name, key string) ([]byte,
 		}
 		return raw, nil
 	} else {
-		return nil, fmt.Errorf("%s not found in %s/%s", key, namespace, name)
+		return nil, fmt.Errorf("%s not found in %s/%s", key, cm.Namespace, cm.Name)
 	}
 }
 
@@ -775,54 +803,36 @@ func (optr *Operator) getCloudConfigFromConfigMap(namespace, name, key string) (
 	if err != nil {
 		return "", err
 	}
+	return getCloudConfigFromConfigMap(cm, key)
+}
+
+func getCloudConfigFromConfigMap(cm *corev1.ConfigMap, key string) (string, error) {
 	if cc, ok := cm.Data[key]; ok {
 		return cc, nil
 	}
-	return "", fmt.Errorf("%s not found in %s/%s", key, namespace, name)
+	return "", fmt.Errorf("%s not found in %s/%s", key, cm.Namespace, cm.Name)
 }
 
 // getGlobalConfig gets global configuration for the cluster, namely, the Infrastructure and Network types.
 // Each type of global configuration is named `cluster` for easy discovery in the cluster.
-func (optr *Operator) getGlobalConfig() (*configv1.Infrastructure, *configv1.Network, *configv1.Proxy, error) {
+func (optr *Operator) getGlobalConfig() (*configv1.Infrastructure, *configv1.Network, *configv1.Proxy, *configv1.DNS, error) {
 	infra, err := optr.infraLister.Get("cluster")
 	if err != nil {
-		return nil, nil, nil, err
+		return nil, nil, nil, nil, err
 	}
 	network, err := optr.networkLister.Get("cluster")
 	if err != nil {
-		return nil, nil, nil, err
+		return nil, nil, nil, nil, err
 	}
 	proxy, err := optr.proxyLister.Get("cluster")
 	if err != nil && !apierrors.IsNotFound(err) {
-		return nil, nil, nil, err
+		return nil, nil, nil, nil, err
 	}
-	return infra, network, proxy, nil
-}
-
-func (optr *Operator) setEtcdOperatorImage(imgs *Images) error {
-	if optr.etcdLister == nil {
-		// if the resource is not found, i.e. it is not created by CVO
-		// which means cluster-etcd-operator images is not part of CVO
-		imgs.ControllerConfigImages.ClusterEtcdOperator = ""
-		glog.V(4).Info("etcd cr not found")
-		return nil
+	dns, err := optr.dnsLister.Get("cluster")
+	if err != nil && !apierrors.IsNotFound(err) {
+		return nil, nil, nil, nil, err
 	}
-	etcd, err := optr.etcdLister.Get("cluster")
-	if err != nil {
-		if apierrors.IsNotFound(err) {
-			imgs.ControllerConfigImages.ClusterEtcdOperator = ""
-			return nil
-		}
-		imgs.ControllerConfigImages.ClusterEtcdOperator = ""
-		return fmt.Errorf("error getting etcd CR: %#v", err)
-	}
-
-	if etcd.Spec.ManagementState == operatorv1.Unmanaged {
-		glog.V(4).Info("etcd cluster in unmanaged")
-		imgs.ControllerConfigImages.ClusterEtcdOperator = ""
-		return nil
-	}
-	return nil
+	return infra, network, proxy, dns, nil
 }
 
 func getRenderConfig(tnamespace, kubeAPIServerServingCA string, ccSpec *mcfgv1.ControllerConfigSpec, imgs *RenderConfigImages, apiServerURL string) *renderConfig {
@@ -859,3 +869,12 @@ func mergeCertWithCABundle(initialBundle, newBundle []byte, subject string) []by
 	}
 	return mergedBytes
 }
+
+func isPoolStatusConditionTrue(pool *mcfgv1.MachineConfigPool, conditionType mcfgv1.MachineConfigPoolConditionType) bool {
+	for _, condition := range pool.Status.Conditions {
+		if condition.Type == conditionType {
+			return condition.Status == corev1.ConditionTrue
+		}
+	}
+	return false
+}
